@@ -125,7 +125,7 @@ start_session_finished (GObject *object, GAsyncResult *result, gpointer data)
     GError *error = NULL;
 
     if (!lightdm_greeter_start_session_finish (greeter, result, &error))
-        status_notify ("%s SESSION-FAILED", greeter_id);
+        status_notify ("%s SESSION-FAILED ERROR=%s", greeter_id, error->message);
     g_clear_error (&error);
 }
 
@@ -135,11 +135,13 @@ write_shared_data_finished (GObject *object, GAsyncResult *result, gpointer data
     LightDMGreeter *greeter = LIGHTDM_GREETER (object);
     gchar *dir, *path, *test_data;
     FILE *f;
+    GError *error = NULL;
 
-    dir = lightdm_greeter_ensure_shared_data_dir_finish (greeter, result);
+    dir = lightdm_greeter_ensure_shared_data_dir_finish (greeter, result, &error);
     if (!dir)
     {
-        status_notify ("%s WRITE-SHARED-DATA ERROR=NO_SHARED_DIR", greeter_id);
+        status_notify ("%s WRITE-SHARED-DATA ERROR=%s", greeter_id, error->message);
+        g_clear_error (&error);
         return;
     }
 
@@ -165,10 +167,11 @@ read_shared_data_finished (GObject *object, GAsyncResult *result, gpointer data)
     gchar *contents = NULL;
     GError *error = NULL;
 
-    dir = lightdm_greeter_ensure_shared_data_dir_finish (greeter, result);
+    dir = lightdm_greeter_ensure_shared_data_dir_finish (greeter, result, &error);
     if (!dir)
     {
-        status_notify ("%s READ-SHARED-DATA ERROR=NO_SHARED_DIR", greeter_id);
+        status_notify ("%s READ-SHARED-DATA ERROR=%s", greeter_id, error->message);
+        g_clear_error (&error);
         return;
     }
 
@@ -185,6 +188,8 @@ read_shared_data_finished (GObject *object, GAsyncResult *result, gpointer data)
 static void
 request_cb (const gchar *name, GHashTable *params)
 {
+    GError *error = NULL;
+
     if (!name)
     {
         g_main_loop_quit (loop);
@@ -195,22 +200,58 @@ request_cb (const gchar *name, GHashTable *params)
         kill (getpid (), SIGSEGV);
 
     else if (strcmp (name, "AUTHENTICATE") == 0)
-        lightdm_greeter_authenticate (greeter, g_hash_table_lookup (params, "USERNAME"));
+    {
+        if (!lightdm_greeter_authenticate (greeter, g_hash_table_lookup (params, "USERNAME"), &error))
+        {
+            status_notify ("%s FAIL-AUTHENTICATE ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "AUTHENTICATE-GUEST") == 0)
-        lightdm_greeter_authenticate_as_guest (greeter);
+    {
+        if (!lightdm_greeter_authenticate_as_guest (greeter, &error))
+        {
+            status_notify ("%s FAIL-AUTHENTICATE-GUEST ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "AUTHENTICATE-AUTOLOGIN") == 0)
-        lightdm_greeter_authenticate_autologin (greeter);
+    {
+        if (!lightdm_greeter_authenticate_autologin (greeter, &error))
+        {
+            status_notify ("%s FAIL-AUTHENTICATE-AUTOLOGIN ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "AUTHENTICATE-REMOTE") == 0)
-        lightdm_greeter_authenticate_remote (greeter, g_hash_table_lookup (params, "SESSION"), NULL);
+    {
+        if (!lightdm_greeter_authenticate_remote (greeter, g_hash_table_lookup (params, "SESSION"), NULL, &error))
+        {
+            status_notify ("%s FAIL-AUTHENTICATE-REMOTE ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "RESPOND") == 0)
-        lightdm_greeter_respond (greeter, g_hash_table_lookup (params, "TEXT"));
+    {
+        if (!lightdm_greeter_respond (greeter, g_hash_table_lookup (params, "TEXT"), &error))
+        {
+            status_notify ("%s FAIL-RESPOND ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "CANCEL-AUTHENTICATION") == 0)
-        lightdm_greeter_cancel_authentication (greeter);
+    {
+        if (!lightdm_greeter_cancel_authentication (greeter, &error))
+        {
+            status_notify ("%s FAIL-CANCEL-AUTHENTICATION ERROR=%s", greeter_id, error->message);
+            g_clear_error (&error);
+        }
+    }
 
     else if (strcmp (name, "START-SESSION") == 0)
         lightdm_greeter_start_session (greeter, g_hash_table_lookup (params, "SESSION"), NULL, start_session_finished, NULL);
@@ -316,6 +357,18 @@ request_cb (const gchar *name, GHashTable *params)
         }
     }
 
+    else if (strcmp (name, "LOG-SESSIONS") == 0)
+    {
+        GList *sessions, *link;
+
+        sessions = lightdm_get_sessions ();
+        for (link = sessions; link; link = link->next)
+        {
+            LightDMSession *session = link->data;
+            status_notify ("%s LOG-SESSION KEY=%s", greeter_id, lightdm_session_get_key (session));
+        }
+    }
+
     else if (strcmp (name, "GET-CAN-SUSPEND") == 0)
     {
         gboolean can_suspend = lightdm_get_can_suspend ();
@@ -393,7 +446,8 @@ connect_finished (GObject *object, GAsyncResult *result, gpointer data)
 
     if (!lightdm_greeter_connect_to_daemon_finish (greeter, result, &error))
     {
-        status_notify ("%s FAIL-CONNECT-DAEMON", greeter_id);
+        status_notify ("%s FAIL-CONNECT-DAEMON ERROR=%s", greeter_id, error->message);
+        g_clear_error (&error);
         exit_code = EXIT_FAILURE;
         g_main_loop_quit (loop);
         return;
@@ -407,7 +461,7 @@ connect_finished (GObject *object, GAsyncResult *result, gpointer data)
 int
 main (int argc, char **argv)
 {
-    gchar *display, *xdg_seat, *xdg_vtnr, *xdg_session_cookie, *xdg_session_class, *mir_socket, *mir_vt, *mir_id, *path;
+    gchar *display, *xdg_seat, *xdg_vtnr, *xdg_session_cookie, *xdg_session_class, *xdg_session_type, *mir_server_host_socket, *mir_vt, *mir_id, *path;
     GString *status_text;
 
 #if !defined(GLIB_VERSION_2_36)
@@ -419,7 +473,8 @@ main (int argc, char **argv)
     xdg_vtnr = getenv ("XDG_VTNR");
     xdg_session_cookie = getenv ("XDG_SESSION_COOKIE");
     xdg_session_class = getenv ("XDG_SESSION_CLASS");
-    mir_socket = getenv ("MIR_SOCKET");
+    xdg_session_type = getenv ("XDG_SESSION_TYPE");  
+    mir_server_host_socket = getenv ("MIR_SERVER_HOST_SOCKET");
     mir_vt = getenv ("MIR_SERVER_VT");
     mir_id = getenv ("MIR_SERVER_NAME");
     if (display)
@@ -431,8 +486,10 @@ main (int argc, char **argv)
     }
     else if (mir_id)
         greeter_id = g_strdup_printf ("GREETER-MIR-%s", mir_id);
-    else if (mir_socket || mir_vt)
+    else if (mir_server_host_socket || mir_vt)
         greeter_id = g_strdup ("GREETER-MIR");
+    else if (g_strcmp0 (xdg_session_type, "wayland") == 0)
+        greeter_id = g_strdup ("GREETER-WAYLAND");
     else
         greeter_id = g_strdup ("GREETER-?");
 
@@ -463,7 +520,7 @@ main (int argc, char **argv)
     g_key_file_load_from_file (config, path, G_KEY_FILE_NONE, NULL);
     g_free (path);
 
-    if (g_key_file_has_key (config, "test-greeter-config", "return-value", NULL))
+    if (g_key_file_get_boolean (config, "test-greeter-config", "exit-on-startup", NULL))    
     {
         int return_value = g_key_file_get_integer (config, "test-greeter-config", "return-value", NULL);
         status_notify ("%s EXIT CODE=%d", greeter_id, return_value);
@@ -504,5 +561,11 @@ main (int argc, char **argv)
 
     g_main_loop_run (loop);
 
+    if (g_key_file_has_key (config, "test-greeter-config", "return-value", NULL))
+    {
+        int return_value = g_key_file_get_integer (config, "test-greeter-config", "return-value", NULL);
+        return return_value;
+    }
+
     return exit_code;
 }
